# -*- coding: utf-8 -*-
from rest_framework import serializers
from thx.api.serializers import (
    ThuxCreateSerializerMixin, ThuxDisableSerializerMixin, ThuxEnableSerializerMixin, ThuxListSerializerMixin,
    ThuxPartialUpdateSerializerMixin, ThuxRetrieveSerializerMixin, ThuxUpdateSerializerMixin,
)

from thx.role.core.serializers import ThuxRoleSerializerMixin

from ... import models


# PROFILE
class ProfileSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_profile'
        model = models.Profile
        fields = '__all__'


class ProfileListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_profile_list'
        model = models.Profile
        fields = '__all__'


class ProfileRetrieveSerializer(ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_profile_retrieve'
        model = models.Profile
        fields = '__all__'


class ProfileCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_profile_create'
        model = models.Profile
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = ProfileRetrieveSerializer


class ProfileUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_profile_update'
        model = models.Profile
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = ProfileRetrieveSerializer


class ProfilePartialUpdateSerializer(
        ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_profile_partial_update'
        model = models.Profile
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = ProfileRetrieveSerializer


class ProfileEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_profile_enable'
        model = models.Profile
        fields = ()
        response_serializer = ProfileRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class ProfileDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_profile_disable'
        model = models.Profile
        fields = ()
        response_serializer = ProfileRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class ProfileLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_profile_light'
        model = models.Profile
        fields = ('id', 'status', 'status_display',)


# ROLE
class RoleSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_role'
        model = models.Role
        fields = '__all__'


class RoleListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_role_list'
        model = models.Role
        fields = '__all__'


class RoleRetrieveSerializer(ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_role_retrieve'
        model = models.Role
        fields = '__all__'


class RoleCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_role_create'
        model = models.Role
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = RoleRetrieveSerializer


class RoleUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_role_update'
        model = models.Role
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = RoleRetrieveSerializer


class RolePartialUpdateSerializer(
        ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_role_partial_update'
        model = models.Role
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = RoleRetrieveSerializer


class RoleEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_role_enable'
        model = models.Role
        fields = ()
        response_serializer = RoleRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class RoleDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_role_disable'
        model = models.Role
        fields = ()
        response_serializer = RoleRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class RoleLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_role_light'
        model = models.Role
        fields = ('id', 'status', 'status_display',)


# MENU TRANSLATION
class MenuTranslationSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_translation'
        model = models.MenuTranslation
        fields = '__all__'


class MenuTranslationListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_translation_list'
        model = models.MenuTranslation
        fields = '__all__'


class MenuTranslationRetrieveSerializer(
        ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_translation_retrieve'
        model = models.MenuTranslation
        fields = '__all__'


class MenuTranslationCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_translation_create'
        model = models.MenuTranslation
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = MenuTranslationRetrieveSerializer


class MenuTranslationUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_translation_update'
        model = models.MenuTranslation
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = MenuTranslationRetrieveSerializer


class MenuTranslationPartialUpdateSerializer(
        ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_translation_partial_update'
        model = models.MenuTranslation
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = MenuTranslationRetrieveSerializer


class MenuTranslationEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_translation_enable'
        model = models.MenuTranslation
        fields = ()
        response_serializer = MenuTranslationRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class MenuTranslationDisableSerializer(
        ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_translation_disable'
        model = models.MenuTranslation
        fields = ()
        response_serializer = MenuTranslationRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class MenuTranslationLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_translation_light'
        model = models.MenuTranslation
        fields = ('id', 'status', 'status_display',)


# MENU
class MenuSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    translations = MenuTranslationSerializer(many=True, read_only=True)
    children = serializers.SerializerMethodField(read_only=True)

    class Meta:
        ref_name = 'v1_role_menu'
        model = models.Menu
        fields = '__all__'

    def get_children(self, instance):
        return MenuSerializer(
            instance=instance.children.filter_user_menu(
                user=self.view.request.user,
                codename=self.view.ar.profile,
                only_root=False
            ),
            context=self.context,
            many=True
        ).data


class MenuListSerializer(MenuSerializer, ThuxListSerializerMixin):
    class Meta(MenuSerializer.Meta):
        ref_name = 'v1_role_menu_list'


class MenuRetrieveSerializer(MenuSerializer, ThuxRetrieveSerializerMixin):
    class Meta(MenuSerializer.Meta):
        ref_name = 'v1_role_menu_retrieve'


class MenuCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_create'
        model = models.Menu
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = MenuRetrieveSerializer


class MenuUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_update'
        model = models.Menu
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = MenuRetrieveSerializer


class MenuPartialUpdateSerializer(
    ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_role_menu_partial_update'
        model = models.Menu
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = MenuRetrieveSerializer


class MenuEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_enable'
        model = models.Menu
        fields = ()
        response_serializer = MenuRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class MenuDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_disable'
        model = models.Menu
        fields = ()
        response_serializer = MenuRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class MenuLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_role_menu_light'
        model = models.Menu
        fields = ('id', 'status', 'status_display',)
